import { AfterContentChecked, AfterViewChecked, Component, Input, OnInit } from '@angular/core';
import { DatasetCrudService } from './services/dataset-crud.service';
import { OpenApi } from '../publishapi/class/openapi';
import { ParameterType } from '../publishapi/class/openapi-enum';
import { OpenApiTag } from '../publishapi/class/openapi-dto';
import { environment } from 'src/environments/environment.prod';
import { NbStepperComponent } from '@nebular/theme';
import { DatasetsDialogComponent } from './datasets-dialog/datasets-dialog.component';
import { MatDialog } from '@angular/material/dialog';
import { HttpClient } from '@angular/common/http';
import { HttpMethod } from '../publishapi/class/http-enum';
import { Observable } from 'rxjs';

interface RequestInfo {
  value?: string;
  result?: string;
  type?: ParameterType;
  tag?: OpenApiTag;
  required: boolean;
}

@Component({
  selector: 'app-datasets',
  templateUrl: './datasets.component.html',
  styleUrls: ['./datasets.component.scss']
})


export class DatasetsComponent implements OnInit, AfterViewChecked, AfterContentChecked {

  @Input() openApi: OpenApi;
  @Input() catalogId: any;
  oldOpenApiStringified: string;

  loading = false;
  ready = false;

  stepper: NbStepperComponent;

  tagEnum = OpenApiTag;

  values: Map<string, Map<string, Map<string, RequestInfo>>>;
  previews: Map<string, any>;
  spinners: Map<string, boolean>;
  FDP_URL = environment.fdpUrl;

  requestHistorical: string[] = [];

  Object = Object;

  //urlrepo = "dataverse.ird.fr";
  //  urlrepo = "data.inrae.fr";

  constructor(
    private dataSetService: DatasetCrudService,
    public dialog: MatDialog,
    private httpClient: HttpClient
  ) { }

  get urlRepo(): string {
    return this.openApi.servers.length > 0 ? this.openApi.servers[0].url : '';
  }

  get openApiHasChanged(): boolean {
    const openApiStringified: string = JSON.stringify(this.openApi);
    if (openApiStringified === this.oldOpenApiStringified) {
      return false;
    } else {
      this.oldOpenApiStringified = openApiStringified;
      return true;
    }
  }

  toggleHistoryText(isOpen: boolean): string {
    if (isOpen) {
      return 'Hide history';
    } else {
      return 'Show history';
    }
  }

  ngOnInit() {
    this.initComponent();
  }

  ngAfterContentChecked() {
    this.initComponent();
  }

  ngAfterViewChecked() {
    window.dispatchEvent(new Event('resize'));
  }

  initComponent() {
    if (!this.openApiHasChanged) {
      return;
    }

    this.loading = false;
    this.ready = false;

    this.values = new Map<string, Map<string, Map<string, RequestInfo>>>();
    this.previews = new Map<string, any>();
    this.spinners = new Map<string, boolean>();

    this.initValueMap();
    this.updateRequestHistorical();
    this.dataSetService.resetDataset();
    this.dataSetService.resetUrls();
  }

  initValueMap() {
    for (const path of this.openApi.paths) {

      if (!this.values.has(path.pathName)) {
        this.values.set(path.pathName, new Map<string, Map<string, RequestInfo>>());
      }

      for (const request of path.requests) {
        if (!this.values.get(path.pathName).has(path.pathName)) {
          this.values.get(path.pathName).set(request.httpmethod, new Map<string, RequestInfo>());
        }

        for (const parameter of request.parameters) {
          if (!this.values.get(path.pathName).get(request.httpmethod).has(parameter.name)) {
            this.values.get(path.pathName)
              .get(request.httpmethod)
              .set(parameter.name,
                {
                  value: parameter.schema.default,
                  type: parameter.in,
                  required: parameter.required
                });
          }
        }
      }
    }
  }
  private getUrls(pathName: string, httpMethod: string, datasetId: any): string {
    const parameters = this.values.get(pathName).get(httpMethod);
    let path = pathName;

    for (const [parameterName, parameterInfo] of parameters) {
      if (parameterInfo.type === ParameterType.path) {
        let value = parameterInfo.value;
        if (parameterInfo.required && datasetId != null) {
          value = datasetId.toString();
        }
        path = path.replace(`{${parameterName}}`, value);
      }
    }
    const myHeaders: any = {};

    const url = new URL(this.urlRepo + path);

    for (const [parameterName, parameterInfo] of parameters) {
      let value = parameterInfo.value;
      if (parameterInfo.required && datasetId != null) {
        value = datasetId.toString();
      }

      switch (parameterInfo.type) {
        case ParameterType.query:
          url.searchParams.set(parameterName, value);
          break;
        case ParameterType.header:
          myHeaders[parameterName] = value;
          break;
        case ParameterType.cookie:
          if (myHeaders.Cookie != null) {
            myHeaders.Cookie = myHeaders.Cookie + `; ${parameterName}=${value}`;
          } else {
            myHeaders.Cookie = `${parameterName}=${value}`;
          }
      }
    }

    const requestHref = url.href;
    return requestHref;

  }

  async launchRequest(pathName: string, httpMethod: string, preview: boolean, datasetId?: number) {
    if (preview) {
      this.spinners.set(pathName, true);
    }

    const parameters = this.values.get(pathName).get(httpMethod);
    let path = pathName;

    for (const [parameterName, parameterInfo] of parameters) {
      if (parameterInfo.type === ParameterType.path) {
        let value = parameterInfo.value;
        if (parameterInfo.required && datasetId != null) {
          value = datasetId.toString();
        }
        path = path.replace(`{${parameterName}}`, value);
      }
    }

    const myHeaders: any = {};

    const url = new URL(this.urlRepo + path);

    for (const [parameterName, parameterInfo] of parameters) {
      let value = parameterInfo.value;
      if (parameterInfo.required && datasetId != null) {
        value = datasetId.toString();
      }

      switch (parameterInfo.type) {
        case ParameterType.query:
          url.searchParams.set(parameterName, value);
          break;
        case ParameterType.header:
          myHeaders[parameterName] = value;
          break;
        case ParameterType.cookie:
          if (myHeaders.Cookie != null) {
            myHeaders.Cookie = myHeaders.Cookie + `; ${parameterName}=${value}`;
          } else {
            myHeaders.Cookie = `${parameterName}=${value}`;
          }
      }
    }

    const requestHref = url.href;
    let result: any = null;

    try {
      const response = await this.getRequestPromiseByHttpMethod(httpMethod as HttpMethod, requestHref, myHeaders, pathName);

      switch (this.getTagByPathName(pathName, httpMethod)) {
        case OpenApiTag.search:
          result = this.findListId(response, pathName, preview);
          this.addRequestInHistorical(requestHref);
          break;
        case OpenApiTag.dataset:
          result = this.processDataSet(response, pathName, preview);
          break;
      }
    } finally {
      this.spinners.set(pathName, false);
    }

    return result;
  }

  getRequestPromiseByHttpMethod(httpMethod: HttpMethod, requestUrl: string, headers: any, pathName: string): Promise<any> {
    let requestObs: Observable<any>;
    console.log(this.openApi.info['x-result'] === "xml")
    switch (httpMethod.toLowerCase()) {
      case HttpMethod.GET:

        if (this.openApi.info['x-result'] === "xml" && this.getTagByPathName(pathName, httpMethod) == OpenApiTag.dataset) {
          requestObs = this.httpClient.get(requestUrl, { headers, responseType: 'text' });
        } else {
          requestObs = this.httpClient.get(requestUrl, { headers });
        }


    }

    return requestObs.toPromise();
  }

  processDataSet(dataSet: any, pathName: string, preview: boolean) {
    if (preview) {
      this.previews.set(pathName, dataSet);
    } else {
      return dataSet;
    }
  }

  findListId(dataSetList: any, pathName: string, preview: boolean) {
    const dataSetIdPath = this.getDatasetIdPath();
    const dataSetIdList: number[] = [];
    this.getIdInResponse(dataSetList, dataSetIdPath, dataSetIdList);

    if (preview) {
      this.previews.set(pathName, dataSetIdList);
    } else {
      return dataSetIdList;
    }
  }

  getIdInResponse(data: any, dataSetIdPath: string[], dataSetIdList: number[]) {
    if (dataSetIdPath.length === 0) {
      return;
    }

    const currentPath = dataSetIdPath[0];

    if (!data.hasOwnProperty(currentPath) || data[currentPath] == null) {
      return;
    }

    if (typeof data[currentPath] !== 'object') {
      dataSetIdList.push(data[currentPath]);
      return;
    }

    if (!Array.isArray(data[currentPath])) {
      this.getIdInResponse(data[currentPath], dataSetIdPath.slice(1), dataSetIdList);
    } else {
      for (const index of data[currentPath]) {
        this.getIdInResponse(index, dataSetIdPath.slice(1), dataSetIdList);
      }
    }
  }

  getDatasetIdPath(): string[] {
    for (const path of this.openApi.paths) {
      for (const request of path.requests) {
        if (request.tags[0] !== OpenApiTag.search) {
          continue;
        }

        for (const response of request.responses) {
          if (!response.httpStatusCode.toString().startsWith('2')) {
            continue;
          }

          for (const content of response.contents) {
            if (content.contentType.toLowerCase() !== 'application/json') {
              continue;
            }

            if (content.example == null || content.example.length === 0) {
              continue;
            }

            return content.example.split('#');
          }
        }
      }
    }

    return [];
  }

  getTagByPathName(pathName: string, httpMethod: string): OpenApiTag {
    for (const path of this.openApi.paths) {
      if (path.pathName !== pathName) {
        continue;
      }

      for (const request of path.requests) {
        if (request.httpmethod === httpMethod) {
          return OpenApiTag[request.tags[0]];
        }
      }
    }

    return null;
  }

  openDatasetsSelectDialog(stepper: NbStepperComponent) {
    this.loading = true;
    this.stepper = stepper;

    let searchRequestPathName: string;
    let searchRequestHttpMethod: string;

    let datasetRequestPathName: string;
    let datasetRequestHttpMethod: string;

    for (const path of this.openApi.paths) {
      for (const request of path.requests) {
        if (request.tags == null || request.tags.length === 0) {
          continue;
        }

        switch (request.tags[0]) {
          case OpenApiTag.search:
            searchRequestPathName = path.pathName;
            searchRequestHttpMethod = request.httpmethod;
            break;
          case OpenApiTag.dataset:
            datasetRequestPathName = path.pathName;
            datasetRequestHttpMethod = request.httpmethod;
            break;
        }
      }
    }

    this.getListFromDialog(searchRequestPathName, searchRequestHttpMethod,
      datasetRequestPathName, datasetRequestHttpMethod);
    /*
      if (response.status.toString().startsWith('2'))
              this.dataSetService.saveDatasets(data);
    */
  }

  private getListFromDialog(searchRequestPathName: string, searchRequestHttpMethod: string,
    datasetRequestPathName: string, datasetRequestHttpMethod: string) {
    this.launchRequest(searchRequestPathName, searchRequestHttpMethod, false).then((response) => {
      this.dialog.open(DatasetsDialogComponent, {
        width: '100vw',
        data: {
          datasetId: response
        },
      }).afterClosed().subscribe({
        next: (datasetId) => this.onCloseDatasetDialog(datasetRequestPathName, datasetRequestHttpMethod, datasetId)
      });
    }).finally(() => this.loading = false);
  }

  private onCloseDatasetDialog(datasetRequestPathName: string, datasetRequestHttpMethod: string, datasetId: number[]) {
    if (datasetId == null || datasetId.length === 0) {
      this.ready = false;
      return;
    }

    this.loading = true;

    const requestPromises: Promise<any>[] = [];


    for (let i = 0; i < datasetId.length; i++) {
      this.dataSetService.saveUrls(this.getUrls(datasetRequestPathName, datasetRequestHttpMethod, datasetId[i]), datasetId[i] );
      let requestPromise: Promise<void>;
      if (this.openApi.info['x-result'] !== "xml") {
        
        if (i < 10) {
          requestPromise = this.launchRequest(datasetRequestPathName, datasetRequestHttpMethod, false, datasetId[i])
            .then((dataset) => {
              this.dataSetService.saveDatasets(dataset, datasetId[i]);
            });
          requestPromises.push(requestPromise);
        }
      }
    }

    Promise.all(requestPromises).then(() => {
      this.ready = true;
      this.stepper.next();
    }).finally(() => this.loading = false);
  }

  private updateRequestHistorical() {
    this.requestHistorical = [];
    const catId = this.openApi.info['x-catalog-id'];

    if (catId == null) {
      return;
    }

    this.dataSetService.getRequestHistorical(catId).subscribe({
      next: (response) => {
        this.requestHistorical = [];
        response.map((el) => { this.requestHistorical.push(el.request); });
      }
    });
  }

  private addRequestInHistorical(request: string) {
    const catId = this.openApi.info['x-catalog-id'];

    if (catId == null) {
      return;
    }

    this.dataSetService.createRequestHistorical(catId, request).subscribe({
      next: () => this.updateRequestHistorical()
    });
  }


}
