import { ViewChild } from '@angular/core';
import { Component, OnInit } from '@angular/core';
import { NbStepperComponent } from '@nebular/theme';
import { cloneDeep } from 'lodash';
import { DatasetCrudService } from '../datasets/services/dataset-crud.service';
import { CatalogService, FdpApiResponseItem } from '../services/catalog.service';
import HttpStatusCode from './class/http-enum';
import { OpenApi, Parameter, Path, Request, Response } from './class/openapi';
import { OpenApiTag } from './class/openapi-dto';
import { ParameterType, ShemaType } from './class/openapi-enum';
import { OpenApiDTOMappingService } from './services/openapi-dto-mapping-service';
import { OpenApiService } from './services/openapi-service';

@Component({
  selector: 'app-publishapi',
  templateUrl: './publishapi.component.html',
  styleUrls: ['./publishapi.component.scss']
})

export class PublishApiComponent implements OnInit {

  @ViewChild('catalogSelect') catalogSelect;
  @ViewChild('datasets') datasets;
  @ViewChild('stepper') stepper: NbStepperComponent;

  constructor(
    private openApiService: OpenApiService,
    private catalogService: CatalogService,
    private openApiDTOMappingService: OpenApiDTOMappingService,
    private datasetService: DatasetCrudService
  ) { }

  openApi: OpenApi;
  parametersTypes: string[];
  shemaTypes: string[];
  httpStatusCodes: string[];
  catalogList: { title: string, catId: string, server: string, type: string, version?: string, description?: string, result?: string}[] = [];
  canNext = false;
  type: string;
  initLabelThree = false;

  ngOnInit(): void {
    this.openApi = new OpenApi(null, null);
    this.parametersTypes = Object.keys(ParameterType);
    this.shemaTypes = Object.keys(ShemaType);
    const httpStatusCodeskeys = Object.keys(HttpStatusCode).filter(k => typeof HttpStatusCode[k as any] === "number");
    this.httpStatusCodes = httpStatusCodeskeys.map(k => HttpStatusCode[k as any]);
    this.initCatalogList();
    this.datasetService.resetDataset();
  }

  getPrettyJson(): string {
    return this.openApiService.getPrettyJson(this.openApi);
  }

  initCatalogList() {
    this.catalogService.getAllCatalogId().subscribe({
      next: (response) => {
        const titlePromises: Promise<any>[] = [];
        response.forEach((catalog) => {
          titlePromises.push(
            
            this.catalogService.getCatalogContentById(catalog.catId).toPromise().then(
              (response2) => {
                this.catalogList.push({
                  catId: catalog.catId,
                  title: this.getTitleFromFdpApi(response2),
                  server: this.getServerFromFdpApi(response2),
                  type: this.getTypeFromFdpApi(response2),
                  description: this.getDescriptionFromFdpApi(response2),
                  version: this.getVersionFromFdpApi(response2),
                });
              }).catch(
                (error) => {
                  if (error.status === 404) {
                    this.catalogService.deleteCatalog(catalog.catId).subscribe();
                  } else {
                    this.catalogList.push({ catId: catalog.catId, title: catalog.catId, server: null, type : null });
                  }
                }
              )
          );
        });
        Promise.all(titlePromises).finally(() => {
          if (this.catalogList.length > 0) {
            this.catalogList.sort((a, b) => a.catId.localeCompare(b.catId));
            this.updateCurrentOpenApi(this.catalogList[0].catId);
          }
        });
      },
      error: (error) => {
        if (error.staus === 404) { /* Ignore */ }
      }
    });
  }

  onChangeRepository(catId: string) {
    this.canNext = false;
    this.openApiService.getByCatalogId(catId).subscribe({
      next: (response) => {
        this.openApi = this.openApiDTOMappingService.mapOpenApiFromDTO(response);
        this.openApi.info['x-catalog-id'] = catId;
        this.openApi.addServers(this.getServer(catId));
        this.type = this.getType(catId);
        this.canNext = true;
      },
      error: () => {
        this.openApi = new OpenApi(catId, this.getServer(catId));
        this.openApi.info['x-format'] = this.getType(catId);
        this.openApi.info['title'] = this.getTitle(catId);
        this.openApi.info['version'] = this.getVersion(catId);
        this.openApi.info['description'] = this.getDescription(catId);
        this.openApi.info['x-result'] = this.getResult(catId);
        this.openApi.paths.push(this.openApiDTOMappingService.getEmptyPath(OpenApiTag.search));
        this.openApi.paths.push(this.openApiDTOMappingService.getEmptyPath(OpenApiTag.dataset));
        this.canNext = true;
      },
    });
  }

  updateCurrentOpenApi(catId: string) {
    this.openApi.info['x-catalog-id'] = catId;
    this.catalogSelect.selected = catId;
    this.onChangeRepository(catId);
  }

  getTitleFromFdpApi(fdpApiResponse: FdpApiResponseItem[]): string {
    for (const item of fdpApiResponse) {
      if (item.predicate.localName === 'title') {
        return item.object.label;
      }
    }
    return '';
  }

  getTypeFromFdpApi(fdpApiResponse: FdpApiResponseItem[]): string {
    for (const item of fdpApiResponse) {
      if (item.predicate.localName === 'keyword') {
        return item.object.label;
      }
    }
    return '';
  }

  getServerFromFdpApi(fdpApiResponse: FdpApiResponseItem[]): string {
    for (const item of fdpApiResponse) {
      if (item.predicate.localName === 'homepage') {
        return item.object.namespace + item.object.localName;
      }
    }
    return '';
  }

  getDescriptionFromFdpApi(fdpApiResponse: FdpApiResponseItem[]): string {
    for (const item of fdpApiResponse) {
      if (item.predicate.localName === 'description') {
        return item.object.label;
      }
    }
    return '';
  }

  getVersionFromFdpApi(fdpApiResponse: FdpApiResponseItem[]): string {
    for (const item of fdpApiResponse) {
      if (item.predicate.localName === 'hasVersion') {
        return item.object.label;
      }
    }
    return '';
  }

  getServer(catId: string): string {
    for (const catalog of this.catalogList) {
      if (catalog.catId === catId) {
        return catalog.server;
      }
    }
    return null;
  } 

  getType(catId: string): string {
    for (const catalog of this.catalogList) {
      if(catalog.catId === catId) {
        return catalog.type;
      }
    }
    return null;
  }

  getTitle(catId: string): string {
    for (const catalog of this.catalogList) {
      if(catalog.catId === catId) {
        return catalog.title;
      }
    }
    return null;
  }

  getVersion(catId: string): string {
    for (const catalog of this.catalogList) {
      if(catalog.catId === catId) {
        return catalog.version;
      }
    }
    return null;
  }

  getDescription(catId: string): string {
    for (const catalog of this.catalogList) {
      if(catalog.catId === catId) {
        return catalog.description;
      }
    }
    return null;
  }

  getResult(catId: string): string {
    for (const catalog of this.catalogList) {
      if(catalog.catId === catId) {
        return (catalog.type === "ISO 19115") ? 'xml' : 'json';
      }
    }
    return null;
  }

  savePublication() {
    this.openApiService.publishOpenApi(this.openApi).subscribe();
  }

  selectAllJson(): void {
    var range = document.createRange();
    range.selectNodeContents(document.getElementById('json'));
    var sel = window.getSelection();
    sel.removeAllRanges();
    sel.addRange(range);
  }

  downloadJson(): void {
    let windowVar: any =  window.navigator;
    const newBlob = new Blob([this.openApiService.getPrettyJson(this.openApi)], { type: 'application/json' });
    if (windowVar && windowVar.msSaveOrOpenBlob) {
      windowVar.msSaveOrOpenBlob(newBlob);
      return;
    }
    const data = window.URL.createObjectURL(newBlob);
    const link = document.createElement('a');
    link.href = data;
    link.download = 'openAPI.json';
    // this is necessary as link.click() does not work on the latest firefox
    link.dispatchEvent(new MouseEvent('click', { bubbles: true, cancelable: true, view: window }));
    setTimeout(() => {
      // For Firefox it is necessary to delay revoking the ObjectURL
      window.URL.revokeObjectURL(data);
      link.remove();
    }, 100);
  }

  /*addPath(): void {
    let emptyPath: Path = this.openApiDTOMappingService.getEmptyPath()
    this.openApi.paths.unshift(emptyPath);
  }*/

  deletePath(path: Path): void {
    const index: number = this.openApi.paths.indexOf(path);
    if (index !== -1) {
      this.openApi.paths.splice(index, 1);
    }
  }

  /*addRequest(path: Path): void {
    let emptyRequest: Request = this.openApiDTOMappingService.getEmptyRequest();
    path.requests.unshift(emptyRequest);
  }*/

  deleteRequest(path: Path, request: Request): void {
    const indexRequest: number = path.requests.indexOf(request);
    if (indexRequest !== -1) {
      path.requests.splice(indexRequest, 1);
    }
  }

  togglRequest(request: Request): void {
    request.toggled = !request.toggled;
  }

  addParameter(request: Request): void {
    let emptyParameter: Parameter = this.openApiDTOMappingService.getEmptyParameter(false, '');
    request.parameters.unshift(emptyParameter);
  }

  duplicateParameter(request: Request, parameter: Parameter): void {
    const newParam = cloneDeep(parameter);
    newParam.required = false;
    request.parameters.unshift(newParam);
  }

  deleteParameter(request: Request, parameter: Parameter): void {
    const indexParameter: number = request.parameters.indexOf(parameter);
    if (indexParameter !== -1) {
      request.parameters.splice(indexParameter, 1);
    }
  }

  addResponse(request: Request): void {
    let emptyResponse: Response = this.openApiDTOMappingService.getEmptyResponse();
    request.responses.unshift(emptyResponse);
  }

  deleteResponse(request: Request, response: Response): void {
    const indexResponse: number = request.responses.indexOf(response);
    if (indexResponse !== -1) {
      request.responses.splice(indexResponse, 1);
    }
  }

  openFileInputDialog(): void {
    document.getElementById('jsonfileinput').click();
  }

  importJson(event: Event): void {
    const files: FileList = (event.target as HTMLInputElement).files;
    const jsonFile: File = files[0];
    var reader = new FileReader();

    reader.onload = () => {
      console.log(reader.result as string);
      const servers = this.openApi.servers;
      const catId = this.openApi.info['x-catalog-id'];

      this.openApi = this.openApiService.getFromString(reader.result as string);
      this.openApi.cleanServers(servers[0].url);
      this.openApi.info['x-catalog-id'] = catId;
    };

    reader.readAsText(jsonFile);
  }

  goToLink() {
    window.open("/swaggerapi");
  }

  duplicatePath(path: Path) {
    const newPath = cloneDeep(path);
    this.openApi.paths.unshift(newPath);
  }

  duplicateRequest(path: Path, request: Request) {
    const newRequest = cloneDeep(request);
    path.requests.unshift(newRequest);
  }

  duplicateResponse(request: Request, response: Response) {
    const newResponse = cloneDeep(response);
    request.responses.unshift(newResponse);
  }

  resetDataset() {
    this.datasetService.resetDataset();
    this.datasetService.resetUrls();
  }

  openDatasetsSelectDialog() {
    this.datasets.openDatasetsSelectDialog(this.stepper);
  }
}

