import { Injectable } from '@angular/core';
import HttpStatusCode, { HttpMethod } from '../class/http-enum';
import { InfoMap, OpenApi, Path, Request, Parameter, Response, Content, ResponseSchema } from '../class/openapi';
import { ContentDTO, HttpCodeToResponseMapDTO, InfoMapDTO, MethodToRequestMapDTO, MimeTypeToContentMapDTO, OpenApiDTO, OpenApiTag, ParameterDTO, PathMapDTO, RequestDTO, ResponseDTO, ResponseSchemaDTO, ServerDTO } from '../class/openapi-dto';
import { ParameterType } from '../class/openapi-enum';

@Injectable({
    providedIn: 'root'
})

export class OpenApiDTOMappingService {

    constructor() { }

    mapContentFromDTO(mimeTypeToContentMapDTO: MimeTypeToContentMapDTO, contentType: string) {
        let contentDTO: ContentDTO = mimeTypeToContentMapDTO[contentType];
        let content: Content = {} as Content;
        content.contentType = contentType;
        content.example = contentDTO.example;
        return content;
    }

    mapResponseFromDTO(httpCodeToResponseMapDTO: HttpCodeToResponseMapDTO, httpStatusCodeToreponseDTOMap: string): Response {
        let responseDTO: ResponseDTO = httpCodeToResponseMapDTO[httpStatusCodeToreponseDTOMap];
        let response: Response = {} as Response;
        response.description = responseDTO.description;
        response.httpStatusCode = httpStatusCodeToreponseDTOMap as unknown as HttpStatusCode;
        response.contents = [] as Content[];
        Object.keys(responseDTO.content).forEach((contentType) => {
            let content: Content = this.mapContentFromDTO(responseDTO.content, contentType);
            response.contents.push(content);
        });
        return response;
    }

    mapParametersFromDTO(parameterDTO: ParameterDTO[]): Parameter[] {
        return parameterDTO as Parameter[];
    }

    mapRequestFromDTO(methodToRequestMapDTO: MethodToRequestMapDTO, httpMethod: HttpMethod): Request {
        let requestDTO: RequestDTO = methodToRequestMapDTO[httpMethod];

        if (requestDTO == null) {
            return this.getEmptyRequest(OpenApiTag.other);
        }

        let request: Request = {} as Request;
        request.httpmethod = httpMethod;
        request.title = requestDTO.title;
        request.summary = requestDTO.summary;
        request.tags = requestDTO.tags as string[];
        request.parameters = this.mapParametersFromDTO(requestDTO.parameters);
        request.responses = [] as Response[];
        request.toggled = false;
        Object.keys(requestDTO.responses).forEach((HttpStatusCode) => {
            let response: Response = this.mapResponseFromDTO(requestDTO.responses, HttpStatusCode);
            request.responses.push(response);
        });
        return request;
    }

    mapPathFromDTO(pathMapDTO: PathMapDTO, pathName: string): Path {
        let methodToRequestMapDTO: MethodToRequestMapDTO = pathMapDTO[pathName];
        let path: Path = {} as Path;
        path.pathName = pathName.trim();
        path.requests = [] as Request[];
        Object.keys(methodToRequestMapDTO).forEach((httpMethod: HttpMethod) => {
            let request: Request = this.mapRequestFromDTO(methodToRequestMapDTO, httpMethod);
            path.requests.push(request);
        });
        return path;
    }

    mapOpenApiFromDTO(openApiDTO: OpenApiDTO): OpenApi {
        const serverUrl = (openApiDTO.servers.length > 0) ? openApiDTO.servers[0].url : ''; 
        let openApi: OpenApi = new OpenApi(openApiDTO.info['x-catalog-id'], serverUrl);
        openApi.info = openApiDTO.info as InfoMap;
        openApi.openapi = openApiDTO.openapi;
        openApi.paths = [] as Path[];
        Object.keys(openApiDTO.paths).forEach((pathName) => {
            let path: Path = this.mapPathFromDTO(openApiDTO.paths, pathName);
            openApi.paths.push(path);
        });
        return openApi;
    }

    /**
     * @param openApi
     */
    mapToDto(openApi: OpenApi): OpenApiDTO {
        let openApiDTO: OpenApiDTO = {} as OpenApiDTO;
        openApiDTO.openapi = openApi.openapi;
        openApiDTO.info = openApi.info as InfoMapDTO;
        openApiDTO.servers = openApi.servers as ServerDTO[];
        openApiDTO.paths = {};
        openApi.paths.forEach(path => {
            openApiDTO.paths[path.pathName] = { };
            path.requests.forEach(request => {
                openApiDTO.paths[path.pathName][request.httpmethod] = {};
                openApiDTO.paths[path.pathName][request.httpmethod].tags = request.tags;
                openApiDTO.paths[path.pathName][request.httpmethod].summary = request.summary;
                openApiDTO.paths[path.pathName][request.httpmethod].parameters = request.parameters as ParameterDTO[];
                openApiDTO.paths[path.pathName][request.httpmethod].responses = {};
                request.responses.forEach(response => {
                    openApiDTO.paths[path.pathName][request.httpmethod].responses[response.httpStatusCode.toString()] = {}
                    openApiDTO.paths[path.pathName][request.httpmethod].responses[response.httpStatusCode.toString()].content = {};
                    openApiDTO.paths[path.pathName][request.httpmethod].responses[response.httpStatusCode.toString()].description = response.description;
                    response.contents.forEach(content => {
                        if (openApiDTO.paths[path.pathName][request.httpmethod].responses[response.httpStatusCode.toString()].content[content.contentType] == null) {
                            openApiDTO.paths[path.pathName][request.httpmethod].responses[response.httpStatusCode.toString()].content[content.contentType] = {};
                        }
                        openApiDTO.paths[path.pathName][request.httpmethod].responses[response.httpStatusCode.toString()].content[content.contentType].example = content.example;
                    });
                });
            });
        });
        return openApiDTO;
    }

    getEmptyPath(tag: OpenApiTag): Path {
        return {
            pathName: '/' + tag,
            requests: [
                this.getEmptyRequest(tag)
            ]
        };
    }

    getEmptyRequest(tag: OpenApiTag): Request {
        const requiredParameter = tag === OpenApiTag.dataset;
        const parameterName = requiredParameter ? 'dataset_id' : '';

        return {
            summary: '',
            httpmethod: HttpMethod.GET,
            tags: [tag],
            title: '',
            parameters: [
                this.getEmptyParameter(requiredParameter, parameterName)
            ],
            responses: [
                this.getEmptyResponse()
            ],
            toggled: false
        };
    }

    getEmptyParameter(requiredParameter: boolean, name: string): Parameter {
        return {
            name,
            in: ParameterType.query,
            schema: {
                type: 'string',
                default: ''
            },
            required: requiredParameter,
            description: ''
        };
    }

    getEmptyResponse(): Response {
        return {
            httpStatusCode: HttpStatusCode.ACCEPTED,
            description: '',
            contents: [{
                contentType: 'application/json',
                example: ''
            }]
        };
    }
}
