import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable, of } from 'rxjs';
import { OpenApi } from '../class/openapi';
import { OpenApiDTO } from '../class/openapi-dto';
import { OpenApiDTOMappingService } from './openapi-dto-mapping-service';
import { environment } from 'src/environments/environment';
import { TokenStorageService } from 'src/app/authentication/services/token-storage.service';

@Injectable({
  providedIn: 'root'
})

export class OpenApiService {
  constructor(
    private http: HttpClient,
    private openApiDTOMappingService: OpenApiDTOMappingService,
    private sessionStorage: TokenStorageService
  ) { }

  get smartApiUrl(): string {
    return environment.smartharvesterUrl + '/harvester/api/publication';
  }

  get tokenHarvesterHeader() {
    return {
      headers: new HttpHeaders({
        Authorization: 'Bearer ' + this.sessionStorage.getUser().token
      })
    };
  }

  getFromRepository(repositoryId: string): Observable<OpenApiDTO> {
    return this.http.get<OpenApiDTO>('/openapi/' + repositoryId);
  }

  /**
   * Replacer for JSON.stringify
   * 
   * Allows to serialize Map
   * @param key 
   * @param value 
   */
  private replacer(key, value) {
    const originalObject = this[key];
    if (originalObject instanceof Map) {
      return [...originalObject].reduce((o, [key, value]) => (o[key] = value, o), {});
    } else {
      return value;
    }
  }

  getPrettyJson(openApi: OpenApi): string {
    let openApiDTO: OpenApiDTO = this.openApiDTOMappingService.mapToDto(openApi);
    return JSON.stringify(openApiDTO, this.replacer, 2);
  }

  getFromJson(): OpenApi {
    let openApiDTO: OpenApiDTO = <OpenApiDTO>JSON.parse(this.jsonFull);
    return this.openApiDTOMappingService.mapOpenApiFromDTO(openApiDTO);
  }

  getFromString(json: string): OpenApi {
    let openApiDTO: OpenApiDTO = <OpenApiDTO>JSON.parse(json);
    return this.openApiDTOMappingService.mapOpenApiFromDTO(openApiDTO);
  }

  getByCatalogId(catId: string): Observable<OpenApiDTO> {
    if (this.sessionStorage.getUser().token == null) {
      return of();
    }

    return this.http.get<OpenApiDTO>(this.smartApiUrl + '/catalogs/' + catId, this.tokenHarvesterHeader);
  }

  publishOpenApi(openApi: OpenApi) {
    if (this.sessionStorage.getUser().token == null) {
      return of();
    }

    const openApiDTO = this.openApiDTOMappingService.mapToDto(openApi);
    return this.http.post(this.smartApiUrl + '/publish', openApiDTO, this.tokenHarvesterHeader);
  }

  private jsonFull =
    '{' +
    '    "openapi" : "3.0.0",' +
    '    "info" : {' +
    '        "title" : "inrae list datasets",' +
    '        "version" : "0",' +
    '        "x-format" : "dataverse",' +
    '        "x-catalog-id" : null,' +
    '        "x-result" : "json",' +
    '        "x-start-param" : "start",' +
    '        "x-page-param" : "per_page"' +
    '    },' +
    '    "servers" : [ ' +
    '        {' +
    '            "url" : "https://data.inrae.fr/api"' +
    '        }' +
    '    ],' +
    '    "paths" : {' +
    '        "/search" : {' +
    '            "get" : {' +
    '                "description" : "Returns a list of datasets",' +
    '                "tags" : [ ' +
    '                    "datasetlist"' +
    '                ],' +
    '                "parameters" : [ ' +
    '                    {' +
    '                        "in" : "query",' +
    '                        "name" : "q",' +
    '                        "schema" : {' +
    '                            "type" : "string",' +
    '                            "default" : "*"' +
    '                        },' +
    '                        "required" : true,' +
    '                        "description" : "for all  star"' +
    '                    }, ' +
    '                    {' +
    '                        "in" : "query",' +
    '                        "name" : "per_page",' +
    '                        "schema" : {' +
    '                            "type" : "integer",' +
    '                            "default" : 10' +
    '                        },' +
    '                        "required" : true,' +
    '                        "description" : "pages"' +
    '                    }, ' +
    '                    {' +
    '                        "in" : "query",' +
    '                        "name" : "type",' +
    '                        "schema" : {' +
    '                            "type" : "string",' +
    '                            "default" : "dataset"' +
    '                        },' +
    '                        "required" : true,' +
    '                        "description" : "dataset"' +
    '                    }, ' +
    '                    {' +
    '                        "in" : "query",' +
    '                        "name" : "start",' +
    '                        "schema" : {' +
    '                            "type" : "integer",' +
    '                            "default" : 0' +
    '                        },' +
    '                        "required" : true,' +
    '                        "description" : "start"' +
    '                    }, ' +
    '                    {' +
    '                        "in" : "query",' +
    '                        "name" : "show_entity_ids",' +
    '                        "schema" : {' +
    '                            "type" : "boolean",' +
    '                            "default" : true' +
    '                        },' +
    '                        "required" : true,' +
    '                        "description" : "id"' +
    '                    }, ' +
    '                    {' +
    '                        "in" : "query",' +
    '                        "name" : "show_my_data",' +
    '                        "schema" : {' +
    '                            "type" : "boolean",' +
    '                            "default" : true' +
    '                        },' +
    '                        "required" : true,' +
    '                        "description" : "aaa"' +
    '                    }' +
    '                ],' +
    '                "responses" : {' +
    '                    "200" : {' +
    '                        "description" : "A JSON array of datasets",' +
    '                        "content" : {' +
    '                            "application/json" : {' +
    '                                "schema" : {' +
    '                                    "type" : "array",' +
    '                                    "items" : {' +
    '                                        "type" : "string"' +
    '                                    }' +
    '                                }' +
    '                            }' +
    '                        }' +
    '                    }' +
    '                }' +
    '            }' +
    '        },' +
    '        "/datasets/{entity_id}" : {' +
    '            "get" : {' +
    '                "description" : "Get the details on the dataset that the identifier is in parameter ",' +
    '                "tags" : [ ' +
    '                    "dataset"' +
    '                ],' +
    '                "parameters" : [ ' +
    '                    {' +
    '                        "in" : "path",' +
    '                        "name" : "entity_id",' +
    '                        "schema" : {' +
    '                            "type" : "string",' +
    '                            "default" : "95707"' +
    '                        },' +
    '                        "required" : true,' +
    '                        "description" : "he Digital Object Identifier (DOI) of the dataset searched."' +
    '                    }' +
    '                ],' +
    '                "responses" : {' +
    '                    "200" : {' +
    '                        "description" : "A JSON array of one dataset",' +
    '                        "content" : {' +
    '                            "application/json" : {' +
    '                                "schema" : {' +
    '                                    "type" : "array",' +
    '                                    "items" : {' +
    '                                        "type" : "string"' +
    '                                    }' +
    '                                }' +
    '                            }' +
    '                        }' +
    '                    }' +
    '                }' +
    '            }' +
    '        }' +
    '    }' +
    '}';

  private jsonEmpty =
    '{' +
    '    "openapi" : "3.0.0",' +
    '    "info" : {' +
    '        "title" : "",' +
    '        "version" : "",' +
    '        "x-format" : "",' +
    '        "x-catalog-id" : "",' +
    '        "x-result" : "",' +
    '        "x-start-param" : "",' +
    '        "x-page-param" : ""' +
    '    },' +
    '    "servers" : [ ' +
    '        {' +
    '            "url" : ""' +
    '        }' +
    '    ],' +
    '    "paths" : {' +
    '        " " : {' +
    '            "get" : {' +
    '                "description" : "",' +
    '                "tags" : [ ' +
    '                    "datasetlist"' +
    '                ],' +
    '                "parameters" : [ ' +
    '                    {' +
    '                        "in" : "query",' +
    '                        "name" : "",' +
    '                        "schema" : {' +
    '                            "type" : "",' +
    '                            "default" : ""' +
    '                        },' +
    '                        "description" : ""' +
    '                    }' +
    '                ],' +
    '                "responses" : {' +
    '                    "200" : {' +
    '                        "description" : "",' +
    '                        "content" : {' +
    '                            " " : {' +
    '                                "schema" : {' +
    '                                    "type" : "array",' +
    '                                    "items" : {' +
    '                                        "type" : "string"' +
    '                                    }' +
    '                                }' +
    '                            }' +
    '                        }' +
    '                    }' +
    '                }' +
    '            }' +
    '        },' +
    '        " " : {' +
    '            "get" : {' +
    '                "description" : "",' +
    '                "tags" : [ ' +
    '                    "dataset"' +
    '                ],' +
    '                "parameters" : [ ' +
    '                    {' +
    '                        "in" : "query",' +
    '                        "name" : "",' +
    '                        "schema" : {' +
    '                            "type" : "",' +
    '                            "default" : ""' +
    '                        },' +
    '                        "description" : ""' +
    '                    }' +
    '                ],' +
    '                "responses" : {' +
    '                    "200" : {' +
    '                        "description" : "",' +
    '                        "content" : {' +
    '                            " " : {' +
    '                                "schema" : {' +
    '                                    "type" : "array",' +
    '                                    "items" : {' +
    '                                        "type" : "string"' +
    '                                    }' +
    '                                }' +
    '                            }' +
    '                        }' +
    '                    }' +
    '                }' +
    '            }' +
    '        }' +
    '    }' +
    '}';

  private emptyResponse =
    '"200" : {' +
    '    "description" : "",' +
    '    "content" : {' +
    '        " " : {' +
    '            "schema" : {' +
    '                "type" : "array",' +
    '                "items" : {' +
    '                    "type" : "string"' +
    '                }' +
    '            }' +
    '        }' +
    '    }' +
    '}';

  private emptyParameter =
    '[' +
    '   {' +
    '      "name" : "",' +
    '      "in" : "query",' +
    '      "schema" : {' +
    '          "type" : "string",' +
    '          "default" : ""' +
    '      }' +
    '      "required" : true,' +
    '      "description" : ""' +
    '   }' +
    ']';


  private emptyMethodJson =
    '   "get" : {' +
    '       "description" : "",' +
    '       "tags" : [ ' +
    '           "datasetlist"' +
    '       ],' +
    '       "parameters" : ' +
    this.emptyParameter +
    '       ,' +
    '       "responses" : {' +
    '           "200" : {' +
    '               "description" : "",' +
    '               "content" : {' +
    '                   "" : {' +
    '                       "schema" : {' +
    '                           "type" : "array",' +
    '                           "items" : {' +
    '                               "type" : "string"' +
    '                           }' +
    '                       }' +
    '                   }' +
    '               }' +
    '           }' +
    '       }' +
    '   }';

  private emptyPathJson = '{' +
    '   " " : {' +
    this.emptyMethodJson +
    '   }' +
    '}';

}
